import os

def log_line(file_path, line):
    _line = ""
    if isinstance(line, list):
        for i in line:
            _line += str(i)
            _line += " | "
        _line = _line[:-2]
    elif isinstance(line, str):
        _line = line
    else:
        print("Unknow input types!")
        return
    
    with open(file_path, 'a+') as f:
        f.write(_line)
        f.write("\n")
        
        
def set_debug_mode():
    os.environ['MUSHAN_DEBUG'] = 1
    
def debug(message):
    if os.environ['MUSHAN_DEBUG'] == 1:
        print(message)


'''
Date: 2022-12-30 21:12:23
LastEditors: mushan wwd137669793@gmail.com
LastEditTime: 2022-12-30 21:17:35
FilePath: /mushan/mushan/dl/loss.py
'''
import torch
import torch.nn as nn


class MMD_loss(nn.Module):
    
    def __init__(self, kernel_mul=2.0, kernel_num=5):
        super(MMD_loss, self).__init__()
        self.kernel_num = kernel_num
        self.kernel_mul = kernel_mul
        self.fix_sigma = None

    def guassian_kernel(self, source, target, kernel_mul=2.0, kernel_num=5, fix_sigma=None):
        n_samples = int(source.size(0))+int(target.size(0))
        total = torch.cat([source, target], dim=0)
        total0 = total.unsqueeze(0).expand(
        int(total.size(0)), int(total.size(0)), int(total.size(1)))
        total1 = total.unsqueeze(1).expand(
        int(total.size(0)), int(total.size(0)), int(total.size(1)))
        L2_distance = ((total0-total1)**2).sum(2)
        if fix_sigma:
            bandwidth = fix_sigma
        else:
            bandwidth = torch.sum(L2_distance.data) / (n_samples**2-n_samples)
        bandwidth /= kernel_mul ** (kernel_num // 2)
        bandwidth_list = [bandwidth * (kernel_mul**i) for i in range(kernel_num)]
        kernel_val = [torch.exp(-L2_distance / bandwidth_temp) for bandwidth_temp in bandwidth_list]
        return sum(kernel_val)

    def forward(self, source, target):
        batch_size = int(source.size()[0])
        kernels = self.guassian_kernel(source, target, kernel_mul=self.kernel_mul, kernel_num=self.kernel_num, fix_sigma=self.fix_sigma)
        XX = kernels[:batch_size, :batch_size]
        YY = kernels[batch_size:, batch_size:]
        XY = kernels[:batch_size, batch_size:]
        YX = kernels[batch_size:, :batch_size]
        loss = torch.mean(XX + YY - XY -YX)
        return loss


from typing import List
import torch
import os
import logging
import pickle


def disable_logger(logger_name: List):
    for n in logger_name:
        logger = logging.getLogger(n)
        logger.propagate = False
        
        
def load_pickle(filename):
    with open(filename, "rb") as f:
        data = pickle.load(f)
    return data

def load_text(filename):
    with open(filename, "r") as f:
        text = f.readlines()
    return text


def dump_pickle(data, filename):
    with open(filename, "wb") as f:
        pickle.dump(data, f)


from time import ctime

from mushan import __version__


def run():
    cur_time = ctime()
    text = f"""
    # package_name
    
    version {__version__} ({cur_time} +0800)
    """
    print(text)



import torch
import librosa
from librosa.filters import mel as librosa_mel_fn
import numpy as np



hann_window = {}
mel_basis = {}

def audio_to_linear(y, n_fft=1024, sr=22050, hop_size=256, win_size=1024, center=False):
    if torch.min(y) < -1.:
        print('min value is ', torch.min(y))
    if torch.max(y) > 1.:
        print('max value is ', torch.max(y))

    global hann_window
    dtype_device = str(y.dtype) + '_' + str(y.device)
    wnsize_dtype_device = str(win_size) + '_' + dtype_device
    if wnsize_dtype_device not in hann_window:
        hann_window[wnsize_dtype_device] = torch.hann_window(win_size).to(dtype=y.dtype, device=y.device)

    y = torch.nn.functional.pad(y.unsqueeze(1), (int((n_fft-hop_size)/2), int((n_fft-hop_size)/2)), mode='reflect')
    y = y.squeeze(1)

    spec = torch.stft(y, n_fft, hop_length=hop_size, win_length=win_size, window=hann_window[wnsize_dtype_device],
                      center=center, pad_mode='reflect', normalized=False, onesided=True, return_complex=False)

    spec = torch.sqrt(spec.pow(2).sum(-1) + 1e-6)
    return spec


def audio_to_mel(y, n_fft=1024, num_mels=80, sampling_rate=22050, hop_size=256, win_size=1024, fmin=0, fmax=8000, center=False):
    if torch.min(y) < -1.:
        print('min value is ', torch.min(y))
    if torch.max(y) > 1.:
        print('max value is ', torch.max(y))

    global mel_basis, hann_window
    dtype_device = str(y.dtype) + '_' + str(y.device)
    fmax_dtype_device = str(fmax) + '_' + dtype_device
    wnsize_dtype_device = str(win_size) + '_' + dtype_device
    if fmax_dtype_device not in mel_basis:
        mel = librosa_mel_fn(sr=sampling_rate, n_fft=n_fft, n_mels=num_mels, fmin=fmin, fmax=fmax)
        mel_basis[fmax_dtype_device] = torch.from_numpy(mel).to(dtype=y.dtype, device=y.device)
    if wnsize_dtype_device not in hann_window:
        hann_window[wnsize_dtype_device] = torch.hann_window(win_size).to(dtype=y.dtype, device=y.device)

    y = torch.nn.functional.pad(y.unsqueeze(1), (int((n_fft-hop_size)/2), int((n_fft-hop_size)/2)), mode='reflect')
    y = y.squeeze(1)

    spec = torch.stft(y, n_fft, hop_length=hop_size, win_length=win_size, window=hann_window[wnsize_dtype_device],
                      center=center, pad_mode='reflect', normalized=False, onesided=True, return_complex=False)

    spec = torch.sqrt(spec.pow(2).sum(-1) + 1e-6)

    spec = torch.matmul(mel_basis[fmax_dtype_device], spec)
    spec = spectral_normalize_torch(spec)

    return spec


def dynamic_range_compression_torch(x, C=1, clip_val=1e-5):
    """
    PARAMS
    ------
    C: compression factor
    """
    return torch.log(torch.clamp(x, min=clip_val) * C)


def dynamic_range_decompression_torch(x, C=1):
    """
    PARAMS
    ------
    C: compression factor used to compress
    """
    return torch.exp(x) / C


def spectral_normalize_torch(magnitudes):
    output = dynamic_range_compression_torch(magnitudes)
    return output


def linear_to_mel(spec, n_fft, num_mels, sr, fmin=0, fmax=None):
    global mel_basis
    dtype_device = str(spec.dtype) + '_' + str(spec.device)
    fmax_dtype_device = str(fmax) + '_' + dtype_device
    if fmax_dtype_device not in mel_basis:
        mel = librosa_mel_fn(sr=sr,
                             n_fft=n_fft,
                             n_mels=num_mels,
                             fmin=fmin,
                             fmax=fmax)
        mel_basis[fmax_dtype_device] = torch.from_numpy(mel).to(dtype=spec.dtype, device=spec.device)
    spec = torch.matmul(mel_basis[fmax_dtype_device], spec)
    spec = spectral_normalize_torch(spec)
    return spec

def get_data(audio, sr=22050, n_fft=1024, hop_size=256, win_size=1024, n_mel=80):

    if audio.max() > 2:
        audio = torch.FloatTensor(audio.astype(np.float32))
        audio_norm = audio / 2 ** 15
        audio_norm = audio_norm.unsqueeze(0)
    else:
        audio = torch.FloatTensor(audio.astype(np.float32))
        audio_norm = audio / 2 ** 15
        audio_norm = audio_norm.unsqueeze(0)

    print(audio_norm.shape)

    spec = audio_to_linear(audio_norm,
                             n_fft=n_fft,
                             sr=sr,
                             hop_size=hop_size,
                             win_size=win_size)

    mel = linear_to_mel(spec,
                            n_fft=n_fft,
                            num_mels=n_mel,
                            sr=sr
                            )

    F0 = librosa.pyin(audio_norm.numpy(),
                      fmin=librosa.note_to_hz('C2'),
                      fmax=librosa.note_to_hz('C7'),
                      sr=22050,
                      frame_length=1024,
                      hop_length=256)[0].squeeze(0)



    return audio_norm, spec.squeeze(0), mel.squeeze(0), F0


from p_tqdm import p_umap
from glob import glob
import os
from scipy.io.wavfile import read
import torch
import numpy as np

def convto2205(ori_audio_path):
    file_name = os.path.basename(ori_audio_path)
    dir_name = os.path.dirname(ori_audio_path)
    input_name = dir_name + '/' + 'new' + file_name
    output_name = ori_audio_path
    os.rename(ori_audio_path, input_name)
    os.system("ffmpeg -i "+ input_name+ " -loglevel fatal -y -ac 1 -ar 22050"+" " + output_name)
    os.remove(input_name)


def torch_load_audio(audio_path, norm=True, sr=None):
    sampling_rate, data = read(audio_path)
    
    if sr != None:
        assert sr == sampling_rate
        
    if norm:
        data = data / 32768.0
        
    data = torch.FloatTensor(data.astype(np.float32))
    

    
    return data, sampling_rate


'''
Date: 2022-07-20 15:22:19
LastEditors: mushan wwd137669793@gmail.com
LastEditTime: 2023-01-05 18:10:43
FilePath: /mushan/setup.py
'''
import os
import sys
from setuptools import setup
from setuptools import find_packages

NAME = "mushan"
AUTHOR = "Mushan"
EMAIL = "wwd137669793@gmail.com"
URL = "None"
LICENSE = "None"
DESCRIPTION = "Personal toolkit."
VERSION = "0.0.5"

if sys.version_info < (3, 6, 0):
    raise RuntimeError(f"{NAME} requires Python >=3.6.0, but yours is {sys.version}!")

__version__ = VERSION

try:
    with open("README.md", encoding="utf8") as f_r:
        _long_description = f_r.read()
except FileNotFoundError:
    _long_description = ""

if __name__ == "__main__":
    setup(
        name=NAME,
        version=__version__,
        author=AUTHOR,
        url="https://github.com/mushanshanshan/mushan-pip",
        author_email=EMAIL,
        Homepage=URL,
        license=LICENSE,
        description=DESCRIPTION,
        packages=find_packages(),
        include_package_data=True,
        setup_requires=["setuptools>=18.0", "wheel"],
        install_requires=open("./requirements.txt", "r").read().splitlines(),
        long_description=_long_description,
        long_description_content_type="text/markdown",
        entry_points={
            "console_scripts": [
                "mushan=mushan.shell:run"
            ]
        },
        package_data={
            "mushan": ["src/*.txt"]
        },
        zip_safe=True,
        classifiers=[
            "Programming Language :: Python :: 3",
            "License :: OSI Approved :: MIT License",
            "Operating System :: OS Independent",
        ],
        python_requires=">=3.6"
    )



'''
Date: 2022-08-11 14:40:52
LastEditors: mushan wwd137669793@gmail.com
LastEditTime: 2022-12-30 21:47:56
FilePath: /mushan/mushan/dl/func.py
'''
import torch
import os
from varname import argname


def get_device():
    
    return torch.device('cuda' if torch.cuda.is_available() else 'cpu')

def disable_cuda(args=None):
    
    
    os.environ["CUDA_VISIBLE_DEVICES"]="-1"
    if torch.cuda.is_available():
        print("Disable CUDA fail!")
    else:
        print("Disable CUDA success!")
        
        
def set_cuda(gpus=None):
    """_summary_

    Args:
        gpus (int, list): _description_
    """
    
    if gpus == None or gpus == -1:
        disable_cuda()
    else:
        _gpus = []
        if isinstance(gpus, list):
            for g in gpus:
                _gpus.append(str(g))
        elif isinstance(gpus, int):
            _gpus.append(str(gpus))
        else:
            print("Unknow input types!")
            return
            
        os.environ["CUDA_VISIBLE_DEVICES"]=",".join(_gpus)
        
        print("Current CUDA Devices: {}".format(torch.cuda.current_device()))
        print("Total Visible CUDA Device Count: {}".format(torch.cuda.device_count()))
    
    
def printShape(*args):
    for i in range(len(args)):
        assert isinstance(args[i], torch.Tensor)
        print(f"{argname(f'args[{i}]')}.shape: {str(list(args[i].shape))}, {str(args[i].dtype)[6:]}")




from mushan import shell
from mushan import audio
from mushan import nb
from mushan import ml
from mushan import dl
from mushan import tool
from mushan import log

__all__ = [
    "__version__",
    "shell",
]



from ..audio.spec_process import get_data
import librosa
import numpy as np
from pathlib import Path

def plot_detail_info_from_file(audio, pitc_scale=1, pitch_mel=True, durations=None, level='f'):
    import matplotlib.pylab as plt
    
    naudio, spec, melspec, F0 = get_data(audio)
    
    fig, ax = plt.subplots(figsize=(20, 4))
    im = ax.imshow(melspec, aspect="auto", origin="lower",
                   interpolation='none')

    freqs = librosa.core.mel_frequencies(fmin=0.0, fmax=8000, n_mels=80)
    if pitch_mel:
        F0_mel = [np.argmin(abs(freqs - p))*pitc_scale for p in F0]
    else:
        F0_mel = F0 * pitc_scale
    
    plt.plot([i for i in range(len(F0_mel))], F0_mel, color='r', linewidth=2.5)

    plt.colorbar(im, ax=ax)
    plt.xlabel("Frames")
    plt.ylabel("Channels")
    plt.tight_layout()

    fig.canvas.draw()
    plt.show()
    
    
def plot_mel_from_data(audio, pitc_scale=1, pitch_mel=True, durations=None, level='f'):
    import matplotlib.pylab as plt
    
    if isinstance(audio,str) or isinstance(audio,Path):
        _, _, melspec, _ = get_data(audio)
        
    fig, ax = plt.subplots(figsize=(20, 4))
    im = ax.imshow(melspec, aspect="auto", origin="lower",
                   interpolation='none')

    plt.colorbar(im, ax=ax)
    plt.xlabel("Frames")
    plt.ylabel("Channels")
    plt.tight_layout()

    fig.canvas.draw()
    plt.show()